/*
 * Copyright (c) 2020, Peter Abeles. All Rights Reserved.
 *
 * This file is part of BoofCV (http://boofcv.org).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package boofcv.abst.filter.binary;

import boofcv.BoofTesting;
import boofcv.alg.filter.binary.GThresholdImageOps;
import boofcv.alg.misc.GImageMiscOps;
import boofcv.core.image.GeneralizedImageOps;
import boofcv.struct.ConfigLength;
import boofcv.struct.image.GrayF32;
import boofcv.struct.image.GrayU8;
import boofcv.struct.image.ImageGray;
import boofcv.struct.image.ImageType;
import boofcv.testing.BoofStandardJUnit;
import org.junit.jupiter.api.Test;

/**
 * @author Peter Abeles
 */
public class TestLocalGaussianBinaryFilter extends BoofStandardJUnit {

	@Test
	public void compare() {
		Class imageTypes[] = new Class[]{GrayU8.class,GrayF32.class};

		for( Class type : imageTypes ) {

			ImageGray input = GeneralizedImageOps.createSingleBand(type,30,40);
			GrayU8 found = new GrayU8(30,40);
			GrayU8 expected = new GrayU8(30,40);

			GImageMiscOps.fillUniform(input,rand,0,200);

			LocalGaussianBinaryFilter alg = new LocalGaussianBinaryFilter(ConfigLength.fixed(9),0.95,true, ImageType.single(type));

			alg.process(input,found);
			GThresholdImageOps.localGaussian(input, expected, ConfigLength.fixed(9), 0.95, true, null, null);

			BoofTesting.assertEquals(found,expected,0);
		}
	}
}
